/**
 * @author    Olaf Braun
 * @copyright 2013-2018 Olaf Braun - Software Development
 * @license   WBB-Elite.de License <https://lizenz.wbb-elite.de/lizenz.html>
 * @package   de.wbb-elite.pushXX.board
 * @category  Push++ WBB4.1
 */
/**
 * Push notification for new thread notifications
 */
Push.Notification.Thread = Push.Notification.extend({
    /**
     * @var number
     */
    _lastPostTime: 0,
    /**
     * @var number
     */
    _pageNo: 0,
    /**
     * @var string
     */
    _defaultMessage: "de.wbb-elite.push++.notification.thread",
    /**
     * @inheritDoc
     */
    init: function (threadID, pageNo, lastPostTime, options) {
        this._objectID = threadID;
        this._lastPostTime = lastPostTime;
        this._pageNo = pageNo;
        this._super($.extend(true, {
            type: "thread"
        }, options || {}));
        new Push.Notification.Thread.Update(threadID, pageNo, options);
    },
    /**
     * @inheritDoc
     */
    _success: function (data) {
        if (data.returnValues.template === undefined || data.returnValues.nextPageNotice === undefined || !data.returnValues.lastPostTime || data.returnValues.templatePageLinks === undefined) return;

        this._lastPostTime = data.returnValues.lastPostTime ? data.returnValues.lastPostTime : this._lastPostTime;

        var threadPostList = $('body#tpl_wbb_thread div.section > .wbbThreadPostList');
        var pagination = $('body#tpl_wbb_thread .messageListPagination');
        var quickReplay = elById('messageQuickReply');
        // remove link
        $('.wbbMorePostsNotice').remove();
        // insert posts
        var template = $(data.returnValues.template);
        template.each(function (key, value) {
            var postID = elData(elBySel('article', this), 'post-id');
            if (elById('post' + postID)) {
                delete template[key];
            }
        });

        if (template.find('li').length > 0) {
            this._showMessage();
            if (quickReplay) {
                if (pagination.length) template.insertBefore(pagination);
                else template.insertBefore(quickReplay);
            } else {
                if (pagination.length) template.insertBefore(pagination);
                else threadPostList.append(template);
            }
            this._showNotification(elById(elAttr(template[0], 'id')));
        }
        if (data.returnValues.nextPageNotice) {
            if (quickReplay) {
                $('<li class="wbbMorePostsNotice"><p class="info">' + data.returnValues.nextPageNotice + '</p></li>').insertBefore(quickReplay);
            } else {
                threadPostList.append($('<li class="wbbMorePostsNotice"><p class="info">' + data.returnValues.nextPageNotice + '</p></li>'));
            }
        }
        if (quickReplay) elData(quickReplay, 'last-post-time', data.returnValues.lastPostTime);

        if (pagination.length > 0) {
            $(pagination.find('.pagination')).html($(data.returnValues.templatePageLinks).html());
            WCF.System.PageNavigation.init('.pagination');
        }
    },
    /**
     * @inheritDoc
     */
    _reload: function () {
        var quickReplay = elById('messageQuickReply');
        if (quickReplay) {
            this._lastPostTime = this._lastPostTime > parseInt(elData(quickReplay, 'last-post-time')) ? this._lastPostTime : parseInt(elData(quickReplay, 'last-post-time'));
        }
        // send request
        this._proxy.setOption('data', {
            actionName: 'getNewPosts',
            className: 'wbb\\data\\thread\\ThreadPushAction',
            objectIDs: [this._objectID],
            parameters: {
                countPosts: false,
                lastPostTime: this._lastPostTime,
                pageNo: this._pageNo
            }
        });
        this._proxy.sendRequest();
    }
});


/**
 * Push notification for update thread posts
 */
Push.Notification.Thread.Update = Push.Notification.extend({
    /**
     * @var number
     */
    pageNo: 0,
    /**
     * @var Array
     */
    _messageIDs: [],
    /**
     * @var String
     */
    _defaultMessage: 'de.wbb-elite.push++.notification.thread.update',

    /**
     * @inheritDoc
     */
    init: function (threadID, pageNo, options) {
        this._objectID = threadID;
        this._pageNo = pageNo;
        this._super($.extend(true, {
            type: "postUpdate"
        }, options || {}));
    },
    /**
     * @inheritDoc
     */
    _success: function (data) {
        var self = this;
        var postID = data.returnValues.postID;
        require(['WoltLabSuite/Core/Dom/Util', 'Dom/Traverse','EventHandler', 'WoltLabSuite/Forum/Controller/Thread'], (function (DomUtil, DomTraverse,EventHandler, ThreadManager) {
            var inlineEditor = ThreadManager.getPostInlineEditor();
            
            var activeElement = elBySel('article.wbbPost', elById('post' + postID));

            var elementData = inlineEditor._elements.get(activeElement);
            var attachmentLists = elBySelAll('.attachmentThumbnailList, .attachmentFileList', elementData.messageFooter);

            // set new content
            DomUtil.setInnerHtml(DomTraverse.childByClass(elementData.messageBody, 'messageText'), data.returnValues.message);

            // handle attachment list
            if (typeof data.returnValues.attachmentList === 'string') {
                for (var i = 0, length = attachmentLists.length; i < length; i++) {
                    elRemove(attachmentLists[i]);
                }

                var element = elCreate('div');
                DomUtil.setInnerHtml(element, data.returnValues.attachmentList);

                var node;
                while (element.childNodes.length) {
                    node = element.childNodes[element.childNodes.length - 1];
                    elementData.messageFooter.insertBefore(node, elementData.messageFooter.firstChild);
                }
            }

            // handle poll
            if (typeof data.returnValues.poll === 'string') {
                // find current poll
                var poll = elBySel('.pollContainer', elementData.messageBody);
                if (poll !== null) {
                    // poll contain is wrapped inside `.jsInlineEditorHideContent`
                    elRemove(poll.parentNode);
                }

                var pollContainer = elCreate('div');
                pollContainer.className = 'jsInlineEditorHideContent';
                DomUtil.setInnerHtml(pollContainer, data.returnValues.poll);

                DomUtil.prepend(pollContainer, elementData.messageBody);
            }

            inlineEditor._updateHistory(inlineEditor._getHash(postID));

            self._showMessage();
        }).bind(this));
    },
    /**
     * @inheritDoc
     */
    _reload: function () {
        this._proxy.setOption('data', {
            actionName: 'reloadPostMessage',
            className: 'wbb\\data\\thread\\ThreadPushAction',
            parameters: {
                pageNo: this._pageNo,
                index: parseInt(elBySel('ul.messageQuickOptions > li > a', elById('post' + this._data.data["messageID"])).innerHTML.replace("#", "")),
                postID: this._data.data["postID"]
            },
            objectIDs: [this._objectID]
        });
        this._proxy.sendRequest();
    }
});